/*
 * Copyright 2010-2011 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jsp.tagext;

import com.squeakysand.commons.lang.StringUtils;

import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.SimpleTagSupport;

public abstract class EnhancedSimpleTagSupport extends SimpleTagSupport {

    protected enum AttributeCombination {
        AT_MOST_ONE, AT_LEAST_ONE, EXACTLY_ONE, ALL, ALL_OR_NONE;
    }

    protected void validateNotNullAttribute(String attributeName, Object attributeValue) throws JspException {
        if (attributeValue == null) {
            throw new JspException(String.format("attribute %s must have a non-null runtime value", attributeName));
        }
    }

    protected void validateNotNullOrEmptyAttribute(String attributeName, String attributeValue) throws JspException {
        if (StringUtils.isNullOrEmpty(attributeName)) {
            throw new JspException(String.format("String attribute %s must have a non-null and non-empty runtime value", attributeName));
        }
    }

    protected void validateAttributeCombination(String attribute1name, Object attribute1value, String attribute2name, Object attribute2value, AttributeCombination combination) throws JspException {
        if (StringUtils.isNullOrEmpty(attribute1name)) {
            throw new JspException("attribute1name parameter cannot be null or empty");
        }
        if (StringUtils.isNullOrEmpty(attribute2name)) {
            throw new JspException("attribute2name parameter cannot be null or empty");
        }
        if (combination == null) {
            throw new JspException("combination parameter cannot be null");
        }
        boolean attribute1isNotNull = attribute1value != null;
        boolean attribute2isNotNull = attribute2value != null;
        switch (combination) {
            case AT_MOST_ONE:
                if (attribute1isNotNull && attribute2isNotNull) {
                    throw new JspException(String.format("only 1 of %s or %s attributes can have a runtime value set", attribute1name, attribute2name));
                }
                break;
            case AT_LEAST_ONE:
                if (!attribute1isNotNull && !attribute2isNotNull) {
                    throw new JspException(String.format("at least 1 of %s or %s attributes must have a runtime value set", attribute1name, attribute2name));
                }
                break;
            case EXACTLY_ONE:
                if (!attribute1isNotNull && !attribute2isNotNull) {
                    throw new JspException(String.format("one attribute out of %s or %s must have a runtime value set", attribute1name, attribute2name));
                } else if (attribute1isNotNull && attribute2isNotNull) {
                    throw new JspException(String.format("only one attribute of %s or %s can have a runtime value set", attribute1name, attribute2name));
                }
                break;
            case ALL:
                if (!attribute1isNotNull || !attribute2isNotNull) {
                    throw new JspException(String.format("both attributes {} and {} must have runtime values set", attribute1name, attribute2name));
                }
                break;
            case ALL_OR_NONE:
                if (attribute1isNotNull != attribute2isNotNull) {
                    throw new JspException(String.format("if either {} or {} has a runtime value set, then the other must also have a runtime value set", attribute1name, attribute2name));
                }
                break;
            default:
                throw new JspException(String.format("unknown AttributeCombination value - %s", combination.name()));
        }
    }

    protected JspWriter getJspWriter() {
        return getJspContext().getOut();
    }

    protected PageContext getPageContext() {
        return (PageContext) getJspContext();
    }

    protected ServletRequest getRequest() {
        return getPageContext().getRequest();
    }

    protected ServletResponse getResponse() {
        return getPageContext().getResponse();
    }

    protected HttpSession getSession() {
        return getPageContext().getSession();
    }

    protected Object getServletConfigInitParameter(String name) {
        return getPageContext().getServletConfig().getInitParameter(name);
    }

    protected Object getServletConfigInitParameter(String name, Object defaultValue) {
        Object result = getServletConfigInitParameter(name);
        if (result == null) {
            result = defaultValue;
        }
        return result;
    }

    protected void setAttribtue(String name, Object value) {
        getJspContext().setAttribute(name, value);
    }

    protected void setApplicationAttribute(String name, Object value) {
        getJspContext().setAttribute(name, value, PageContext.APPLICATION_SCOPE);
    }

    protected void setSessionAttribute(String name, Object value) {
        getJspContext().setAttribute(name, value, PageContext.SESSION_SCOPE);
    }

    protected void setRequestAttribute(String name, Object value) {
        getJspContext().setAttribute(name, value, PageContext.REQUEST_SCOPE);
    }

    protected void setPageAttribute(String name, Object value) {
        getJspContext().setAttribute(name, value, PageContext.PAGE_SCOPE);
    }

    protected Object getApplicationAttribute(String name) {
        return getJspContext().getAttribute(name, PageContext.APPLICATION_SCOPE);
    }

    protected Object getSessionAttribute(String name) {
        return getJspContext().getAttribute(name, PageContext.SESSION_SCOPE);
    }

    protected Object getRequestAttribute(String name) {
        return getJspContext().getAttribute(name, PageContext.REQUEST_SCOPE);
    }

    protected Object getPageAttribute(String name) {
        return getJspContext().getAttribute(name, PageContext.PAGE_SCOPE);
    }

    protected Object getApplicationAttribute(String name, Object defaultValue) {
        Object result = getApplicationAttribute(name);
        if (result == null) {
            result = defaultValue;
        }
        return result;
    }

    protected Object getSessionAttribute(String name, Object defaultValue) {
        Object result = getSessionAttribute(name);
        if (result == null) {
            result = defaultValue;
        }
        return result;
    }

    protected Object getRequestAttribute(String name, Object defaultValue) {
        Object result = getRequestAttribute(name);
        if (result == null) {
            result = defaultValue;
        }
        return result;
    }

    protected Object getPageAttribute(String name, Object defaultValue) {
        Object result = getPageAttribute(name);
        if (result == null) {
            result = defaultValue;
        }
        return result;
    }

    protected Object findAttribute(String name) {
        return getJspContext().findAttribute(name);
    }

    protected Object findAttribute(String name, Object defaultValue) {
        Object result = findAttribute(name);
        if (result == null) {
            result = defaultValue;
        }
        return result;
    }

    protected void removeApplicationAttribute(String name) {
        getJspContext().removeAttribute(name, PageContext.APPLICATION_SCOPE);
    }

    protected void removeSessionAttribute(String name) {
        getJspContext().removeAttribute(name, PageContext.SESSION_SCOPE);
    }

    protected void removeRequestAttribute(String name) {
        getJspContext().removeAttribute(name, PageContext.REQUEST_SCOPE);
    }

    protected void removePageAttribute(String name) {
        getJspContext().removeAttribute(name, PageContext.PAGE_SCOPE);
    }

}
