/*
 * Copyright 2010-2011 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jsp.tagext;

import com.squeakysand.commons.io.FileFinder;
import com.squeakysand.commons.lang.SystemProperties;

import java.io.File;
import java.io.IOException;
import java.util.List;

import org.apache.commons.io.filefilter.IOFileFilter;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class TagLibraryUtils {

    private static final Logger LOG = LoggerFactory.getLogger(TagLibraryUtils.class);

    public static Class<?> findTeiClass(Class<?> klass, ClassLoader loader) {
        LOG.trace("findTeiClass called: klass = {}, loader = {}", klass, loader);
        Class<?> teiClass = null;
        String teiClassName = generateDefaultTeiClassName(klass.getName());
        try {
            teiClass = loader.loadClass(teiClassName);
        } catch (ClassNotFoundException ex) {
            LOG.debug("unable to find TEI class with name {}", teiClassName);
        }
        return teiClass;
    }

    public static String findTeiClass(String qualifiedTagClassName, final File srcDir) throws IOException {
        LOG.trace("findTeiClass called: qualifiedTagClassName = {}, srcDir = {}", qualifiedTagClassName, srcDir);
        String result = null;
        String defaultQualifiedTeiClassName = generateDefaultTeiClassName(qualifiedTagClassName);
        final String defaultTeiPackageName = defaultQualifiedTeiClassName.substring(0, defaultQualifiedTeiClassName.lastIndexOf('.'));
        final String defaultTeiClassName = defaultQualifiedTeiClassName.substring(defaultQualifiedTeiClassName.lastIndexOf('.') + 1);
        LOG.debug("tei class info: {} {}", defaultQualifiedTeiClassName, defaultTeiClassName);
        IOFileFilter fileFilter = new IOFileFilter() {

            @Override
            public boolean accept(File file) {
                return accept(file.getParentFile(), file.getName());
            }

            @Override
            public boolean accept(File dir, String name) {
                LOG.debug("accept: {}/{}", dir, name);
                boolean result = false;
                if (name.equals(defaultTeiClassName + ".java")) {
                    try {
                        String srcDirPath = srcDir.getCanonicalPath();
                        LOG.debug("srcDirPath: {}", srcDirPath);
                        String dirPath = dir.getCanonicalPath();
                        LOG.debug("dirPath: {}", dirPath);
                        String trimmedPath = dirPath.substring(srcDirPath.length() + 1);
                        LOG.debug("trimmedPath: {}", trimmedPath);
                        String packageName = trimmedPath.replace(SystemProperties.FILE_SEPARATOR, ".");
                        LOG.debug("packageName: {}", packageName);
                        if (defaultTeiPackageName.equals(packageName)) {
                            result = true;
                        }
                    } catch (IOException e) {
                        LOG.error(e.getMessage(), e);
                    }
                }
                LOG.debug("returning {} from accept", result);
                return result;
            }

        };
        FileFinder fileFinder = new FileFinder(fileFilter);
        List<File> files = fileFinder.findFiles(srcDir);
        if (files != null && files.size() > 0) {
            result = defaultQualifiedTeiClassName;
        }
        LOG.trace("returnining Tei class name: {}", result);
        return result;
    }

    /**
     * Generate default names for tags based on the tag handler class.
     */
    public static String generateDefaultTagName(Class<?> klass) {
        String simpleClassName = klass.getSimpleName();
        return generateDefaultTagName(simpleClassName);
    }

    public static String generateDefaultTagName(String simpleClassName) {
        String result = simpleClassName;
        // lowercase the first letter
        result = Character.toLowerCase(result.charAt(0)) + result.substring(1);
        // if the class result ends in "Tag", then crop it off
        if (result.endsWith("Tag")) {
            result = result.substring(0, result.length() - 3);
        }
        return result;
    }

    public static String generateDefaultTeiClassName(String qualifiedTagClassName) {
        return qualifiedTagClassName + "ExtraInfo";
    }

}
